package src;

import java.awt.*;
import java.util.Random;

public class DrawingBoat {
    // Объект-сущность лодки
    public EntityBoat entityBoat;
    // Объект отрисовки весел лодки
    private DrawingBoatPaddle __drawingBoatPaddle;
    // Координата X верхнего левого угла лодки
    private float __startPosX;
    // Координата Y верхнего левого угла лодки
    private float __startPosY;
    // Ширина отрисовки лодки
    private final int __boatWidth = 100;
    // Высота отрисовки лодки
    private final int __boatHeight = 60;
    // Ширина области отрисовки
    private Integer __pictureWidth = null;
    // Высота области отрисовки
    private Integer __pictureHeight = null;


    // Инициализатор класса
    public void Init(int speed, float weight, Color bodyColor) {
        entityBoat = new EntityBoat();
        entityBoat.Init(speed, weight, bodyColor);
        __drawingBoatPaddle = new DrawingBoatPaddle();
        SetPaddlesCount();
    }

    // Метод для установки количества весел лодки
    public void SetPaddlesCount() {
        Random rnd = new Random();
        int paddlesCount = rnd.nextInt(1, 4);
        __drawingBoatPaddle.SetEnumNumber(paddlesCount);
    }

    // Метод установки позиции и размеров объекта
    public void SetPosition(int x, int y, int width, int height) {
        if (x > 0 && x < width - __boatWidth && y > 0 && y < height - __boatHeight) {
            __startPosX = x;
            __startPosY = y;
            __pictureWidth = width;
            __pictureHeight = height;
        }
    }

    // Метод перемешения обхекта
    public void MoveTransport(EnumDirection enumDirection) {
        // Если не установлен объект-сущность - выходим
        if (entityBoat == null) {
            return;
        }

        // Если не назначены границы области отрисовки - выходим
        if (__pictureWidth == null || __pictureHeight == null) {
            return;
        }

        // Проверяем возможность перемещения и перемещаем обхект по заданному направлению
        switch (enumDirection) {
            case Right:
                if (__startPosX + __boatWidth + entityBoat.Step() < __pictureWidth) {
                    __startPosX += entityBoat.Step();
                }
                break;
            case Left:
                if (__startPosX - entityBoat.Step() > 0) {
                    __startPosX -= entityBoat.Step();
                }
                break;
            case Up:
                if (__startPosY - entityBoat.Step() > 0) {
                    __startPosY -= entityBoat.Step();
                }
                break;
            case Down:
                if (__startPosY + __boatHeight + entityBoat.Step() < __pictureHeight) {
                    __startPosY += entityBoat.Step();
                }
                break;
        }
    }

    // Метод отрисовки объекта
    public void DrawTransport(Graphics g) {
        // Если не установлен объект-сущность - выходим
        if (entityBoat == null) {
            return;
        }

        // Если координаты не валидны или размер области отрисовки не установлен - выходим
        if (__startPosX < 0 || __startPosY < 0
                || __pictureHeight == null || __pictureWidth == null) {
            return;
        }

        Graphics2D g2d = (Graphics2D) g;

        int height = __boatHeight - 30;

        // Промежуточные переменные, чтобы не писать каждый раз (int)
        int startX = (int) __startPosX;
        int startY = (int) __startPosY + 15;

        // Рисуем корпус лодки
        // Задаем цвет корпуса
        g2d.setColor(entityBoat.BodyColor);

        // Массив координат X для полигона корпуса лодки
        int[] xPoints = new int[]{
                startX,
                startX + __boatWidth - __boatWidth / 4,
                startX + __boatWidth,
                startX + __boatWidth - __boatWidth / 4,
                startX
        };

        // Массив координат Y для полигона корпуса лодки
        int[] yPoints = new int[]{
                startY,
                startY,
                startY + height / 2,
                startY + height,
                startY + height
        };

        // Заполняем полигон
        g2d.fillPolygon(xPoints, yPoints, 5);

        // Рисуем окантовку
        g2d.setColor(Color.BLACK);
        g2d.drawPolygon(xPoints, yPoints, 5);

        // Рисуем палубу
        // Левая дуга
        g2d.drawArc(
                startX + __boatWidth / 10,
                startY + height / 5,
                height - height / 5 - height / 5,
                height - height / 5 - height / 5,
                90, 180
        );

        // Правая дуга
        g2d.drawArc(
                startX + __boatWidth - __boatWidth / 4 - __boatWidth / 10 - height / 5,
                startY + height / 5,
                height - height / 5 - height / 5,
                height - height / 5 - height / 5,
                -90, 180
        );

        // Верхняя линия
        g2d.drawLine(
                startX + __boatWidth / 10 + (height - height / 2) / 2,
                startY + height / 5,
                startX + __boatWidth - __boatWidth / 4 - __boatWidth / 10 - height / 5 + (height - height / 2) / 2,
                startY + height / 5
        );

        // Нижняя линия
        g2d.drawLine(
                startX + __boatWidth / 10 + (height - height / 2) / 2,
                startY + height - height / 5,
                startX + __boatWidth - __boatWidth / 4 - __boatWidth / 10 - height / 5 + (height - height / 2) / 2,
                startY + height - height / 5
        );

        // Отрисовка весел лодки
        __drawingBoatPaddle.DrawBoatPaddles(g, entityBoat.BodyColor, __startPosX, startY, __boatWidth, height);
    }

    // Метод изменения границ области отрисовки
    public void ChangeBorders(int width, int height) {
        __pictureWidth = width;
        __pictureHeight = height;

        // Если новые размеры области отрисовки меньше, чем размеры отрисовки лодки,
        // то обнуляем размеры области отрисовки - рисовать не можем
        if (__pictureWidth <= __boatWidth || __pictureHeight <= __boatHeight) {
            __pictureWidth = null;
            __pictureHeight = null;
            return;
        }

        // Если выходим за правую границу, то сдвигаемся влево
        if (__startPosX + __boatWidth > __pictureWidth) {
            __startPosX = __pictureWidth - __boatWidth;
        }

        // Если выходим за нижнюю границу, то сдвигаемся вверх
        if (__startPosY + __boatHeight > __pictureHeight) {
            __startPosY = __pictureHeight - __boatHeight;
        }
    }
}
